/*********************************************//** @file HepMCVisualElements.cxx
*
* Implementation of the graphics elements
*
* Interactive viewer for the HepMC data record: define a number of classes for
* vertices and particles that know how to draw and expand themselves.         
*
* @author <A HREF="mailto:sboeser@hep.ucl.ac.uk">Sebastian B&ouml;ser</A>
* @date 16.05.07
*
*//****************************************************************************/
   
#include "HepMCVisual/HepMCVisualElements.h"

//we have added our classes to namespace HepMC
using namespace HepMC;

/*************************/
/*     VisualParticle    */
/*************************/

//Fully qualified constructor
VisualParticle::VisualParticle(
  const HepMC::FourVector& momentum,
  int pdg_id,
  int status,
  const HepMC::Flow& itsflow,
  const HepMC::Polarization& polar)
: GenParticle(momentum, pdg_id, status, itsflow, polar), TAttLine(1,kSolid,1) {

  //Initialize
  Initialize();
}

//Fully qualified constructor
VisualParticle::VisualParticle(const HepMC::GenParticle& particle)
: GenParticle(particle), TAttLine(1,kSolid,1) {
  //Initialize
  Initialize();
}

//Default initialization for all constructors
void VisualParticle::Initialize(){

  //Set proper dimensions for end nibbles
  fNibbleXRad=0.014; fNibbleYRad=0.007;
  
  //Set start and end nibbles
  fStartNibble=TEllipse(0,0,fNibbleXRad,fNibbleYRad,0,360,0);
  fEndNibble=TEllipse(0,0,fNibbleXRad,fNibbleYRad,0,360,0);

  //Nibbles are always solid
  fStartNibble.SetFillStyle(1001);
  fEndNibble.SetFillStyle(1001);

  //Only create info popup when its needed
  fInfoFrame = 0;

  //Set default line style
  SetDefaultStyle();

  //Put negative default so we know we have not been drawn before
  fLine = TLine(-1,-1,-1,-1);

  //Set default length for unconnected particles
  fDefaultLength=0.1;

  //Do not show particle by default
  fShow= kFALSE;

  //Do not show particles labels by default
  fStaticLabel= kFALSE;

  //Do not reconnect particle by default
  fReconnect = kFALSE;

  //Set default label attributes
  fAttLabel = TAttText(13,0,1,0,2*fNibbleXRad);
  fAttLabel.SetTextFont(42);

}

//Default destructor
VisualParticle::~VisualParticle(){

  //destroy Info-Popup if still there
  if (fInfoFrame) {
    fInfoFrame->UnmapWindow();
    gClient->UnregisterPopup(fInfoFrame);
    delete fInfoFrame;
  }
}

//
//---- Get and Set methods ----
//

Double_t VisualParticle::GetStartX(){

  //If possible, get start from production vertex
  if (IsVisual(production_vertex()))
    return Visualize(production_vertex())->GetX();
  else 
    return fLine.GetX1();
}

Double_t VisualParticle::GetStartY(){

  //If possible, get start from production vertex
  if (IsVisual(production_vertex()))
    return Visualize(production_vertex())->GetY();
  else 
    return fLine.GetY1();
}

Double_t VisualParticle::GetEndX(){
  //If possible, get end from end vertex
  if (IsVisual(end_vertex()))
    return Visualize(end_vertex())->GetX();
  else 
    return fLine.GetX2();
}

Double_t VisualParticle::GetEndY(){
  //If possible, get end from end vertex
  if (IsVisual(end_vertex()))
    return Visualize(end_vertex())->GetY();
  else 
    return fLine.GetY2();
}

Double_t VisualParticle::GetDir(){
  //the theta angle of the particle track
  Double_t dX=GetEndX()-GetStartX();
  Double_t dY=GetEndY()-GetStartY();

  //if particles are not initialized, they are straight up/down;
  if ( fabs(dX) < 1e-8 ) return TMath::Sign(1.,dY)*90;
  
  Double_t theta= 180./TMath::Pi()*TMath::ATan(dY/dX);
  return theta;
}

  
EType_t VisualParticle::GetParticleType(){
  
  //NOTE:: This will classify
  // - diquarks as baryons
  // - technicolor as mesons
  // - susy as other
  
  //Quarks are 1-8
  if ( abs(pdg_id()) < 11) return kQuark;
  //Leptons are 11-18
  if ( abs(pdg_id()) < 21 ) return kLepton;
  //Gauge bosons are usually below 40
  if ( abs(pdg_id()) < 30 ) return kGauge;
  //baryons have non-zero fourth digit (from right)
  if ( (abs(pdg_id())/1000)%10 > 0 ) return kBaryon;
  //mesons have non-zero third digit (from right)
  if ( (abs(pdg_id())/100)%10 > 0 ) return kMeson;
  //unknown group
  return kOther;

}

Bool_t VisualParticle::GetDrawn(){
  
  //Check if particle is actually Drawn on the pad
  if (gPad->FindObject(this)) return kTRUE;
    else return kFALSE;
}

void VisualParticle::SetDefaultStyle(){

  //Set the line style dependant on particle type
  switch (GetParticleType()){
    case kQuark:
      //Set line color for particles and antiparticles
      if (pdg_id()<0) SetLineColor(9);
        else SetLineColor(4);
      //quarks are straight solid lines
      SetLineStyle(kSolid);
      SetLineWidth(1);
      break ;
    case kLepton:
      //Set line color for particles and antiparticles
      if (pdg_id()<0) SetLineColor(8);
        else SetLineColor(3);
      //leptons are green straight lines, apart form neutrinos that are dotted
      if (pdg_id()%2==0) SetLineStyle(kDotted);
        else SetLineStyle(kSolid);
      SetLineWidth(1);
      break ;
    case kGauge:
      //Gauge bosons are red solid lines
      SetLineColor(2);
      SetLineStyle(kSolid);
      SetLineWidth(1);
      break ;
    case kMeson:
      //Set line color for particles and antiparticles
      if (pdg_id()<0) SetLineColor(9);
        else SetLineColor(4);
      //Mesons are solid thick lines
      SetLineStyle(kSolid);
      SetLineWidth(2);
      break ;
    case kBaryon:
      //Set line color for particles and antiparticles
      if (pdg_id()<0) SetLineColor(9);
        else SetLineColor(4);
      //Baryons are solid thicker lines
      SetLineStyle(kSolid);
      SetLineWidth(3);
      break ;
    case kOther:
      //Everything else is a black dash-dotted line
      SetLineColor(1);
      SetLineStyle(kDashDotted);
      SetLineWidth(1);
      break ;
    default :
      cerr << "ERROR: Unknown particle type: " << GetParticleType() << endl;
  }
}
 
std::vector<VisualParticle*> VisualParticle::GetSiblings(){

  std::vector<VisualParticle*> sibs;

  //we need both vertices
  if (! ( production_vertex() && end_vertex() )) return sibs;

  //Loop over all children from the same parent vertex
  for ( GenVertex::particle_iterator par
        = production_vertex()->particles_begin(HepMC::children);
        par != production_vertex()->particles_end(HepMC::children); ++par ) 

    //Check for visual particles
    if (IsVisual(*par))
      //that are not ourselves
      if (Visualize(*par) != this )
        //that are actually part of the pad
        if (Visualize(*par)->GetDrawn())
          //That have a visual end vertex
          if (IsVisual((*par)->end_vertex()))
            //that is the same as ours
            if ((*par)->end_vertex() == end_vertex()){
              //Add to vector
              sibs.push_back(Visualize(*par));
            }

  return sibs;

}

 
//return either of the vertices, 0 if there is none
GenVertex* VisualParticle::AnyVertex(){
  
  //check for production vertex
  if (production_vertex()) return production_vertex();
  //check for end vertex
  if (end_vertex()) return end_vertex();

  //return 0 if there is no vertex
  return 0;
}

//
//---- Graphics methods ----
//

void VisualParticle::Draw(Option_t* option){

  //check for vertices
  if (!AnyVertex()){
    cerr << "ERROR: Cannot draw particles without vertex! -- Ignored!" << endl;
    return ;
  }

  //Initialize line
  if (!InitializeLine()){
    cerr << "ERROR: Failed to initialize particle! -- Ignored!" << endl;
    return ;
  }

  //Add ourselves to the Pad
  AppendPad(option);
}

void VisualParticle::Undraw(){
  
  //Remove myself from the Pad
  gPad->RecursiveRemove(this);

}

void VisualParticle::RecursiveRemove(TObject* obj){

  //Paranoia check
  if (obj){ 
    
    //Check if one of our vertices has gone
    if (IsVisual(end_vertex()))
      if (obj == Visualize(end_vertex())){
        //when still connected
        if (IsVisual(production_vertex()))
          //Unshow myself
          fShow = kFALSE;
        else
          //remove me from pad
          Undraw();
      }

    //Check if one of our vertices has gone
    if (IsVisual(production_vertex()))
      if (obj == Visualize(production_vertex())){
        //when still connected
        if (IsVisual(end_vertex()))
          //Unshow myself
          fShow = kFALSE;
        else
          //remove me from pad
          Undraw();
      }
  }
}

Bool_t VisualParticle::InitializeLine(){

  //check for one vertex only
  if (IsVisual(end_vertex())) if (IsVisual(production_vertex())) return kTRUE;
  
//  //check if we maybe already have reasonable values
//  if ( (GetStartX() > 0) && (GetStartY() > 0) &&
//      (GetEndX() > 0) && (GetEndY() > 0) )
//        return kTRUE;
//  
  //Get start position from end vertex
  if (IsVisual(end_vertex())) {
    Double_t theta=Visualize(end_vertex())->GetNewParentDir();
    Double_t startx =
      GetEndX() - fDefaultLength*TMath::Cos(theta/180.*TMath::Pi());
    Double_t starty =
      GetEndY() - fDefaultLength*TMath::Sin(theta/180.*TMath::Pi());
    fLine = TLine(startx, starty, GetEndX(),GetEndY());
    return kTRUE;
  }
  
  //Get end position from start vertex
  if (IsVisual(production_vertex())) {
    Double_t theta=Visualize(production_vertex())->GetNewChildDir();
    Double_t endx =
      GetStartX() + fDefaultLength*TMath::Cos(theta/180.*TMath::Pi());
    Double_t endy =
      GetStartY() + fDefaultLength*TMath::Sin(theta/180.*TMath::Pi());
    fLine = TLine(GetStartX(), GetStartY(), endx,endy);
    return kTRUE;
  }

  return kFALSE;
}

void VisualParticle::PaintLine(Option_t* option){

  //Gluons and photons are painted seperatly
  if ( pdg_id()==21 ){
    //Start and end at other side of nibble
    Double_t dX= fStartNibble.GetX1() - GetStartX();
    Double_t dY= fStartNibble.GetY1() - GetStartY();
    //gluons are curly lines
    TCurlyLine fCLine(GetStartX()+2*dX,GetStartY()+2*dY,
                      GetEndX()-2*dX,GetEndY()-2*dY,fNibbleXRad,fNibbleXRad/2);
    fCLine.SetCurly();
    //Set line attributes
    fCLine.SetLineWidth(GetLineWidth());
    fCLine.SetLineColor(GetLineColor());
    fCLine.SetLineStyle(GetLineStyle());
    //Apply changes and paint
    fCLine.Paint(option);
  } else if ( pdg_id()==22){
    //Start and end at other side of nibble
    Double_t dX= fStartNibble.GetX1() - GetStartX();
    Double_t dY= fStartNibble.GetY1() - GetStartY();
    //photons are wavy lines
    TCurlyLine fCLine(GetStartX()+2*dX,GetStartY()+2*dY,
                      GetEndX()-2*dX,GetEndY()-2*dY,fNibbleXRad,fNibbleXRad/2);
    fCLine.SetLineWidth(GetLineWidth());
    fCLine.SetLineColor(GetLineColor());
    fCLine.SetLineStyle(GetLineStyle());
    fCLine.SetWavy();
    //Apply changes and paint
    fCLine.Paint(option);
  } else {
    //Set line attributes
    fLine.SetLineWidth(GetLineWidth());
    fLine.SetLineColor(GetLineColor());
    fLine.SetLineStyle(GetLineStyle());
    //Apply changes and paint
    fLine.Paint();
  }
}

void VisualParticle::PaintStartNibble(Option_t* option){

  //Set proper color
  fStartNibble.SetFillColor(GetLineColor());
  fStartNibble.SetLineColor(GetLineColor());
  
  //Get proper nibble position
  Double_t NRad2 = fNibbleXRad*fNibbleXRad;
  Double_t dX=GetEndX()-GetStartX();
  Double_t dY=GetEndY()-GetStartY();

  //shift ellipse center along line
  if ( fabs(dX) > fabs(dY) ){
    //more horizontal lines
    Double_t dYdX2 = (dY/dX)*(dY/dX);
    //Nibbles are always inward
    if ( dX > 0 ) {
      fStartNibble.SetX1( GetStartX()+TMath::Sqrt(NRad2/(1+dYdX2)));
    } else {
      fStartNibble.SetX1( GetStartX()-TMath::Sqrt(NRad2/(1+dYdX2))); 
    }
    //Nibbles are always inward
    if ( dY > 0 ) {
      fStartNibble.SetY1( GetStartY()+TMath::Sqrt(dYdX2*NRad2/(1+dYdX2)));
    } else {
      fStartNibble.SetY1( GetStartY()-TMath::Sqrt(dYdX2*NRad2/(1+dYdX2)));
    }
  } else {
    //more vertical lines
    Double_t dXdY2 = (dX/dY)*(dX/dY);
    if ( dX > 0 ) {
      fStartNibble.SetX1( GetStartX()+sqrt(dXdY2*NRad2/(1+dXdY2)));
    } else {
      fStartNibble.SetX1( GetStartX()-sqrt(dXdY2*NRad2/(1+dXdY2)));
    }
    //Nibbles are always inward
    if ( dY > 0 ) {
      fStartNibble.SetY1( GetStartY()+sqrt(NRad2/(1+dXdY2)));
    } else {
      fStartNibble.SetY1( GetStartY()-sqrt(NRad2/(1+dXdY2)));
    }
  }
  //Get theta from the slope
  fStartNibble.SetTheta( GetDir() );

  //Finally, paint the nibble
  fStartNibble.Paint(option);
}

void VisualParticle::PaintEndNibble(Option_t* option){

  //Set proper color
  fEndNibble.SetFillColor(GetLineColor());
  fEndNibble.SetLineColor(GetLineColor());

  //Get proper nibble position
  Double_t NRad2 = fNibbleXRad*fNibbleXRad;
  Double_t dX=GetEndX()-GetStartX();
  Double_t dY=GetEndY()-GetStartY();

  //shift ellipse center along line
  if ( fabs(dX) > fabs(dY) ){
    //more horizontal lines
    Double_t dYdX2 = (dY/dX)*(dY/dX);
    //Nibbles are always inward
    if ( dX > 0 ) {
      fEndNibble.SetX1( GetEndX()-TMath::Sqrt(NRad2/(1+dYdX2)));
    } else {
      fEndNibble.SetX1( GetEndX()+TMath::Sqrt(NRad2/(1+dYdX2))); 
    }
    //Nibbles are always inward
    if ( dY > 0 ) {
      fEndNibble.SetY1( GetEndY()-TMath::Sqrt(dYdX2*NRad2/(1+dYdX2)));
    } else {
      fEndNibble.SetY1( GetEndY()+TMath::Sqrt(dYdX2*NRad2/(1+dYdX2)));
    }
  } else {
    //more vertical lines
    Double_t dXdY2 = (dX/dY)*(dX/dY);
    if ( dX > 0 ) {
      fEndNibble.SetX1( GetEndX()-sqrt(dXdY2*NRad2/(1+dXdY2)));
    } else {
      fEndNibble.SetX1( GetEndX()+sqrt(dXdY2*NRad2/(1+dXdY2)));
    }
    //Nibbles are always inward
    if ( dY > 0 ) {
      fEndNibble.SetY1( GetEndY()-sqrt(NRad2/(1+dXdY2)));
    } else {
      fEndNibble.SetY1( GetEndY()+sqrt(NRad2/(1+dXdY2)));
    }
  }
  //Get theta from the slope
  fEndNibble.SetTheta( GetDir() );
//  fEndNibble.SetTheta( 180./TMath::Pi()*TMath::ATan(dY/dX) );

  //Finally paint it
  fEndNibble.Paint(option);
}

void VisualParticle::Paint(Option_t* option){

  //Make sure line has a minimum length for single-sided particles
  if ((! production_vertex()) || (! end_vertex())){
    //Get current length
    Double_t dX = GetEndX()-GetStartX();
    Double_t dY = GetEndY()-GetStartY();
    Double_t len = TMath::Sqrt(dX*dX + dY*dY);
    //Get lenght ratio to minimum length
    Double_t stretch = 6*fNibbleXRad/len;
    if (stretch > 1.)
      //stretch line to minium length, keep direction
      if ( production_vertex() ){
        //stretch at end of line
        fLine.SetX2( GetStartX()+ (stretch*dX));
        fLine.SetY2( GetStartY()+ (stretch*dY));
      } else {
        //stretch at begining of line
        fLine.SetX1( GetEndX()- (stretch*dX));
        fLine.SetY1( GetEndY()- (stretch*dY));
      }
  }
    
 
  // all particles always also exist as straight lines
  fLine = TLine(GetStartX(),GetStartY(),GetEndX(), GetEndY());

  //Draw both of them if particle is shown
  if (fShow) {
    //Draw nibbles first
    PaintStartNibble(option);
    PaintEndNibble(option);
    //Draw line last - gluons and photons need nibble centers
    PaintLine(option);
    //Paint the particles name in the middle of the two nibbles
    Double_t xCenter = (GetEndX()-GetStartX())/2+GetStartX();
    Double_t yCenter = (GetEndY()-GetStartY())/2+GetStartY();
    //If the particles label shall always be shown
    if (GetStaticLabel())
      PaintLabel(gPad->XtoPixel(xCenter),gPad->YtoPixel(yCenter));
  } else {
   //only paint if corresponding vertex is shown
   if (IsVisual(end_vertex()))
     if ( Visualize(end_vertex())->GetDrawn())
       PaintEndNibble(option);
   if (IsVisual(production_vertex()))
     if ( Visualize(production_vertex())->GetDrawn())
       PaintStartNibble(option);
  }
}


TGCompositeFrame * VisualParticle::GetInfoFrame(){

  //if there is not InfoFrame create one
  if (!fInfoFrame) MakeInfoFrame();

  return fInfoFrame;
}

void VisualParticle::MakeInfoFrame(){
  
  //Delete old popup if existing
  if (fInfoFrame) {
    fInfoFrame->UnmapWindow();
    gClient->UnregisterPopup(fInfoFrame);
    delete fInfoFrame;
  }
   
// --- Old style ---
//  //Get window that belongs to this pad
//  // 1) get window handle
//  Window_t hCanvas = gVirtualX->GetWindowID(gPad->GetCanvasID());
//  // 2) get window pointer from that
//  TGWindow* pCanvas = gClient->GetWindowById(hCanvas);
//  - in this case, the InfoFrame window is cut off at the border of the Canvas

// --- New style ---
  //Create a new info popup frame as child of graphics Root
  fInfoFrame = new TGCompositeFrame(gClient->GetRoot(),160,80,kRaisedFrame|kDoubleBorder);

  //Change win attributes
  SetWindowAttributes_t wattr;
  wattr.fMask             = kWAOverrideRedirect | kWASaveUnder;
  wattr.fOverrideRedirect = kTRUE;
  wattr.fSaveUnder        = kTRUE;
  gVirtualX->ChangeWindowAttributes(fInfoFrame->GetId(),&wattr);
  
  // Try to get ParticleData:
  ParticleData* pData = 0;
  // See if we can get the parent event of any of these
  if (IsVisual(AnyVertex()->parent_event()))
    //Try to get particle info
    pData = (Visualize(AnyVertex()->parent_event()))->GetParticleData(pdg_id());

  //Some generic layouts
  TGLayoutHints * fExpL = 
    new TGLayoutHints(kLHintsExpandX | kLHintsExpandY, 4, 4, 2, 0);
  TGLayoutHints * fLeftL =
    new TGLayoutHints(kLHintsLeft | kLHintsExpandY , 0, 0, 2, 0);
  TGLayoutHints * fRightL =
    new TGLayoutHints(kLHintsRight | kLHintsExpandY , 0, 0, 2, 0);
  TGLayoutHints * fExpHL = 
    new TGLayoutHints(kLHintsExpandX, 4, 4, 2, 0);

  //Keep track of size and width of the added particles
  UInt_t h = 0; 
  UInt_t w = 0;
  UInt_t tbh = fLeftL->GetPadBottom() + fLeftL->GetPadTop();
  
  //Add a frame for the HepMC lines
  TGGroupFrame * fVert = new TGGroupFrame(fInfoFrame,"HepMC Info");
  fInfoFrame->AddFrame(fVert,fExpL);

  //Add a horizontal frame to add ID and barcode
  TGHorizontalFrame *fIDBar = new TGHorizontalFrame(fVert);
  fVert->AddFrame(fIDBar, new TGLayoutHints( kLHintsExpandX | kLHintsExpandY,
                                             0, 0, 0, 0));
  
  //Add particle ID
  TString pid("ID:"); pid += TString(Form(" %i",pdg_id()));
  TGLabel* fID = new TGLabel(fIDBar,pid.Data());
  fIDBar->AddFrame(fID,fLeftL);

  //Add particle barcode
  TString barc("Barcode:"); barc += TString(Form(" %i",barcode()));
  TGLabel* fBar = new TGLabel(fIDBar,barc.Data());
  fIDBar->AddFrame(fBar,fRightL);

  h += fID->GetDefaultHeight()+tbh; 
  w = TMath::Max(fID->GetDefaultWidth()+fBar->GetDefaultWidth(),w);
  
  //Add particle status
  TString pstat("Status:"); pstat += TString(Form(" %i",status()));
  TGLabel* fStat = new TGLabel(fVert,pstat.Data());
  fVert->AddFrame(fStat,fLeftL);
  h += fStat->GetDefaultHeight()+tbh; 
  w = TMath::Max(fStat->GetDefaultWidth(),w);

  //Add particle energy
  TString en(Form("Energy: %.2f [GeV]",momentum().e()/1e3));
  TGLabel* fEn = new TGLabel(fVert,en.Data());
  fVert->AddFrame(fEn,fLeftL);
  h += fEn->GetDefaultHeight()+tbh; 
  w = TMath::Max(fEn->GetDefaultWidth(),w);

  //Add momentum
  TString mom(Form("Momentum: %.2f [GeV]",momentum().mag()/1e3));
  TGLabel* fmom = new TGLabel(fVert,mom.Data());
  fVert->AddFrame(fmom,fLeftL);
  h += fmom->GetDefaultHeight()+tbh; 
  w = TMath::Max(fmom->GetDefaultWidth(),w);

  //Add Pt
  TString pt(Form("Transv. Momentum: %.2f [GeV]",momentum().perp()/1e3));
  TGLabel* fpt = new TGLabel(fVert,pt.Data());
  fVert->AddFrame(fpt,fLeftL);
  h += fpt->GetDefaultHeight()+tbh; 
  w = TMath::Max(fpt->GetDefaultWidth(),w);

  //Add calculated mass
  TString cmass(Form("Calc. Mass: %.2f [MeV]",momentum().m()));
  TGLabel* fcmass = new TGLabel(fVert,cmass.Data());
  fVert->AddFrame(fcmass,fLeftL);
  h += fcmass->GetDefaultHeight() +tbh; 
  w = TMath::Max(fcmass->GetDefaultWidth(),w);

  //Add eth and phi
  TGHorizontalFrame* fetaphi = new TGHorizontalFrame(fVert);
  fVert->AddFrame(fetaphi, new TGLayoutHints(kLHintsExpandX|kLHintsExpandY,0,0,0,0));
  
  TGLabel* fetal = new TGLabel(fetaphi,"h: ");
#ifdef WITH_ROOT_LOCAL_FONT_BUG
  fetal->SetTextFont("*-symbol-*-*-*-14-*",kTRUE);
#else
  fetal->SetTextFont("*-symbol-*-*-*-14-*",kFALSE);
#endif
  fetaphi->AddFrame(fetal,fLeftL);

  TString eta(Form("%.2f",momentum().eta()));
  if (momentum().eta() > 1e5) eta = "+inf";
  if (momentum().eta() < -1e5) eta = "-inf";
  TGLabel* feta = new TGLabel(fetaphi,eta.Data());
  fetaphi->AddFrame(feta,fLeftL);

  TString phi(Form("%.2f [rad]",momentum().phi()));
  TGLabel* fphi = new TGLabel(fetaphi,phi.Data());
  fetaphi->AddFrame(fphi,fRightL);

  TGLabel* fphil = new TGLabel(fetaphi,"f: ");
#ifdef WITH_ROOT_LOCAL_FONT_BUG
  fphil->SetTextFont("*-symbol-*-*-*-14-*",kTRUE);
#else
  fphil->SetTextFont("*-symbol-*-*-*-14-*",kFALSE);
#endif
  fetaphi->AddFrame(fphil,fRightL);

  h += feta->GetDefaultHeight() +tbh; 

  if (pData) {

    //Add a frame for the PDT lines
    TGGroupFrame * fPDT = new TGGroupFrame(fInfoFrame,"PDT Info");
    fInfoFrame->AddFrame(fPDT,fExpHL);

    TString sname("Name: "); sname += TString(pData->name());
    TGLabel* fName = new TGLabel(fPDT,sname.Data());
    fPDT->AddFrame(fName,fLeftL);
    h += fName->GetDefaultHeight() +tbh; 
    w = TMath::Max(fName->GetDefaultWidth(),w);

    TString smass(Form("Mass: %.2f [MeV]",pData->mass()*1.e3));
    TGLabel* fMass = new TGLabel(fPDT,smass);
    fPDT->AddFrame(fMass,fLeftL);
    h += fMass->GetDefaultHeight() +tbh; 
    w = TMath::Max(fMass->GetDefaultWidth(),w);

    TString swidth(Form("Width: %.2f [MeV]",pData->width()*1.e3));
    TGLabel* fWidth = new TGLabel(fPDT,swidth);
    fPDT->AddFrame(fWidth,fLeftL);
    h += fWidth->GetDefaultHeight() +tbh; 
    w = TMath::Max(fWidth->GetDefaultWidth(),w);

    //Add a horizontal frame to add spin and charge
    TGHorizontalFrame *fSpinCharge = new TGHorizontalFrame(fPDT);
    fPDT->AddFrame(fSpinCharge,
      new TGLayoutHints( kLHintsExpandX | kLHintsExpandY, 0, 0, 0, 0));

    //Add spin
    TString spin(Form("Spin: %i",pData->spin()));
    TGLabel* fspin = new TGLabel(fSpinCharge,spin.Data());
    fSpinCharge->AddFrame(fspin,fLeftL);

    //Add charge
    TString charge(Form("Charge: %.2f",pData->charge()));
    TGLabel* fCharge = new TGLabel(fSpinCharge,charge.Data());
    fSpinCharge->AddFrame(fCharge,fRightL);
    h += fSpinCharge->GetDefaultHeight() +tbh; 
    w = TMath::Max(fSpinCharge->GetDefaultWidth(),w);
  }

  //Account for borders
  w += fExpL->GetPadLeft() + fExpL->GetPadRight()
       + fLeftL->GetPadLeft() + fLeftL->GetPadRight()
       + 2*fInfoFrame->GetBorderWidth();
  h += fExpL->GetPadTop() + fExpL->GetPadBottom()
            + 2*fInfoFrame->GetBorderWidth()
            + fID->GetDefaultHeight() + tbh;  //group frame text
  if (pData)
    h += fExpHL->GetPadTop() + fExpHL->GetPadBottom()
            + fID->GetDefaultHeight() + tbh;  //group frame text

  //Text is not all at left, don't know why
  fInfoFrame->Layout();
  w += 2*fIDBar->GetX();

  //Also acount for TGFrame extra height
  h += 2*fVert->GetY();
  if (pData) h += 2*fVert->GetY(); 

  //Adapt popup size to text size
  fInfoFrame->Resize(w,h);

}

//Show the info frame of this particle and all other 
//particles inbetween the same vertices
void VisualParticle::ShowInfoFrame(Int_t px, Int_t py){

     Int_t PadPixelX, PadPixelY;
     UInt_t PadPixelW, PadPixelH;
     //Get the x/y position of the current frame
     gVirtualX->GetGeometry(gPad->GetCanvasID(),
                            PadPixelX,PadPixelY,PadPixelW,PadPixelH);
     //Draw the info box
     MakeInfoFrame();
     //put at the proper position in the frame
     //fInfoFrame->Move(px+10,py+10);
     fInfoFrame->Move(px+PadPixelX,py+PadPixelY);
     fInfoFrame->Layout();
     fInfoFrame->MapSubwindows();
     fInfoFrame->MapRaised();
     gClient->RegisterPopup(fInfoFrame);

}

//Paint a small label showing the particle ID
void VisualParticle::PaintLabel(Int_t px, Int_t py){

  //Change to current text attributes
  fAttLabel.Modify();
  //Default label is just particle ID
  std::string label(Form("PID: %i",pdg_id()));
  //Check if we can get an name from the parent event
  if (IsVisual(AnyVertex()->parent_event())){
    std::string name;
    name =(Visualize(AnyVertex()->parent_event()))->GetParticleName(pdg_id());
    if (!name.empty()) label=name;
  }
  //Paint the label
  gVirtualX->DrawText(px,py,0,1,&label[0],TVirtualX::kOpaque);
  //Update the canvas to show it
  //gPad->Update();

}


//
//---- User interaction methods
//

Int_t VisualParticle::DistancetoPrimitive(Int_t px, Int_t py){

  //Always use TLine distance (never TCurlyLine)
  Int_t dLine = fLine.DistancetoPrimitive(px,py);

  //Start and end Nibbles also count if existing
  Int_t dENibble = 9999;
  Int_t dEVertex = 9999;
  if (IsVisual(end_vertex())){
    dEVertex = Visualize(end_vertex())->DistancetoPrimitive(px,py);
    dENibble = fEndNibble.DistancetoPrimitive(px,py);
  }
  Int_t dSNibble = 9999;
  Int_t dSVertex = 9999;
  if (IsVisual(production_vertex())){
    dSVertex = Visualize(production_vertex())->DistancetoPrimitive(px,py);
    dSNibble = fStartNibble.DistancetoPrimitive(px,py);
  }

  //Vertices have precedence (otherwise line obscures vertex markers)
  if (( dEVertex <= 10 ) || ( dSVertex <= 10 )) return 9999;

  //Then come the start and end nibbles
  if ( dSNibble < dLine ) return dSNibble;
  if ( dENibble < dLine ) return dENibble;

  //Else return the line, but only if shown
  if (GetShow()) return dLine;
    else return TMath::Min(dSNibble,dENibble);
}

void VisualParticle::ExecuteEvent(Int_t event, Int_t px, Int_t py)
{
   // Execute action corresponding to one event.
   // This member function is called when a particle is clicked with the
   // locator

   // If closer than this, the nibbles get it
   const Int_t kMinNibbleDist = 3;

   //Pixel offset for label and info-Box from cursor location
   const Int_t pXOffset = 10;
   const Int_t pYOffset = 7;

   Double_t x=0; Double_t y=0;
   x = gPad->AbsPixeltoX(px);
   y = gPad->AbsPixeltoY(py);

   //Get siblings (other particles inbetween the same vertices)
   std::vector<VisualParticle*> sibs = GetSiblings();

   switch (event) {

   case kMouseEnter:
     //Set nice cursor when over object
     gPad->SetCursor(kHand);
     break ;
   
   case kMouseLeave:
     //Revert to old cursor when away
     //should be done by canvas, not me?!
     //gPad->SetCursor(kPointer);

     //Redraw pad to remove old label
     gPad->Modified(kTRUE);
     break ;

   case kMouseMotion:

     //Redraw pad to remove old labels
     gPad->Draw();

     //If the particles label is not always shown
     if (! GetStaticLabel())
       //Show the label
       PaintLabel(px + pXOffset, py + pYOffset);
       //Loop over siblings
       for (UInt_t iSib = 0; iSib < sibs.size(); iSib++)
         //show their label as well
         sibs.at(iSib)->PaintLabel(px + pXOffset, py + pYOffset + (iSib+1)*10);
       //Update the pad
       gPad->Update();

     //Indicate the user can pick the particle
     gPad->SetCursor(kHand);

     //Check if we are close to one of the ends
     if (fEndNibble.DistancetoPrimitive(px,py) < kMinNibbleDist ) {
       if (end_vertex()) gPad->SetCursor(kLeftSide);
         else gPad->SetCursor(kMove);
     }
     if (fStartNibble.DistancetoPrimitive(px,py) < kMinNibbleDist ) {
       if (production_vertex()) gPad->SetCursor(kRightSide);
         else gPad->SetCursor(kMove);
     }
     
     break ;

   case kButton1Down:
     //Check if we are close to one of the ends
     if (fEndNibble.DistancetoPrimitive(px,py) < kMinNibbleDist )
       if (IsVisual(end_vertex())) {
         ///\todo Handle end nibble for particles inbetween same vertices
         // Toggle shown bit 
         fShow = fShow ? kFALSE : kTRUE;
         //Loop over siblings
         for (UInt_t iSib = 0; iSib < sibs.size(); iSib++)
           //toggle their shown bit as well
           if (sibs.at(iSib)->GetShow()) sibs.at(iSib)->SetShow(kFALSE);
             else sibs.at(iSib)->SetShow(kTRUE);
         //Temporary protect end vertex -- no IsVisual check needed
         Visualize(end_vertex())->SetProtect(kTRUE);
         //Update on parent side
         UpdateParents();
         //Done with update - unprotect
         Visualize(end_vertex())->SetProtect(kFALSE);
         return ;
       } else {
         // lock in LineMove mode
         fMovingLine = kTRUE;
         // pass event to TLine
         fLine.ExecuteEvent(event,px,py);
         return ;
       }
     if (fStartNibble.DistancetoPrimitive(px,py) < kMinNibbleDist )
       if (IsVisual(production_vertex())){
         ///\todo Handle start nibble for particles inbetween same vertices
         // Toggle shown bit
         fShow = fShow ? kFALSE : kTRUE;
         //Loop over siblings
         for (UInt_t iSib = 0; iSib < sibs.size(); iSib++)
           //toggle their shown bit as well
           if (sibs.at(iSib)->GetShow()) sibs.at(iSib)->SetShow(kFALSE);
             else sibs.at(iSib)->SetShow(kTRUE);
         //Temporary protect production vertex
         Visualize(production_vertex())->SetProtect(kTRUE);
         // Update on children side
         UpdateChildren();
         //Done with update - unprotect
         Visualize(production_vertex())->SetProtect(kFALSE);
         return ;
       } else {
         // lock in LineMove mode
         fMovingLine = kTRUE;
         // pass event to TLine
         fLine.ExecuteEvent(event,px,py);
         return ;
       }
   
     //Otherwise show the particle info box
     
     //Redraw to remove old labels
     gPad->Modified(kTRUE);
     gPad->Update();
     
     //Indicate the user can pick the particle
     gPad->SetCursor(kHand);
     
     //Show the info frame
     ShowInfoFrame(px + pXOffset, py + pYOffset);

     //Check for siblings
     if ( sibs.size() > 0 ){

       //Calculate number of columns
       Int_t nCol = int(sqrt(double(sibs.size()+1)))+2;

       //keep track of upper right corner of the last frame
       Int_t pX = px + pXOffset + fInfoFrame->GetWidth(); 
       Int_t pY = py + pYOffset ;
       
       //also keep track of the row height
       Int_t pYRow = fInfoFrame->GetHeight();

       //Loop over siblings to show their info frame
       for (UInt_t iSib = 0; iSib < sibs.size(); iSib++){
         
         //Start a new row if necessary
         if (((iSib+1)%nCol)==0){
           pX = px + pXOffset; //reset px
           pY = pY + pYRow;    //add to py
         }
         
         //Draw siblings info frame
         sibs.at(iSib)->ShowInfoFrame(pX, pY);

         //Update position
         pX += sibs.at(iSib)->GetInfoFrame()->GetWidth();
         Int_t CurFrameHeight = sibs.at(iSib)->GetInfoFrame()->GetHeight();
         pYRow = CurFrameHeight > pYRow ? CurFrameHeight : pYRow ;
       }
     }
     break ;

   case kButton1Motion:
     //Give to line if in LineMoving mode
     if (fMovingLine) fLine.ExecuteEvent(event,px,py);
     //We do not move the info box with the mouse button
     break ;

   case kButton1Up:

    //If in LineMoving mode, finish it
    if (fMovingLine) {
      fMovingLine = kFALSE;
      fLine.ExecuteEvent(event,px,py);
    }

    //If there is an InfoFrame, get rid of it
    if (fInfoFrame) {
      fInfoFrame->UnmapWindow();
      gClient->UnregisterPopup(fInfoFrame);
    }

    //Check for siblings
    if ( sibs.size() > 0 )
      //Loop over siblings 
      for (UInt_t iSib = 0; iSib < sibs.size(); iSib++)
        //to unshow their info frame
        if (sibs.at(iSib)->GetInfoFrame()) {
          sibs.at(iSib)->GetInfoFrame()->UnmapWindow();
          gClient->UnregisterPopup(sibs.at(iSib)->GetInfoFrame());
        }

         
    //Finally, mark the pad for an update
    gPad->Modified(kTRUE);
    break ;

   case kButton1Locate:
     //Give to line if in LineMoving mode
     if (fMovingLine) fLine.ExecuteEvent(event,px,py);
     break ;

   case kButton1Double:
     //Toggle the static label flag
     fStaticLabel = fStaticLabel ? kFALSE : kTRUE;
     
  }
}

// Loops in UpdateParents and UpdateChildren are avoided by a check for
// IsDrawn() on the particle in VisualVertex::Draw. We do not check for
// IsDrawn() on the vertex, since new particles may have been added to the
// vertex in the meantime, while a particle always only has two ends.
void VisualParticle::UpdateParents(){

  // Fold in and out on production side
  if (production_vertex()){
    
    //Visualize parent if needed
    VisualVertex* vvtx = Visualize(production_vertex());
    
    if (GetShow()) {
      //Draw parent
      vvtx->Draw();
    } else {
      //Order is important here
      vvtx->UndrawParents();
      vvtx->Undraw();
    }
  }
}

void VisualParticle::UpdateChildren(){

  // Fold in and out on end side
  if (end_vertex()){
    
    //Visualize child if needed
    VisualVertex* vvtx = Visualize(end_vertex());
    
    if (GetShow()) {
      //Draw child
      vvtx->Draw();
    } else {
      //Order is important here
      vvtx->UndrawChildren();
      vvtx->Undraw();
    }
  }
}

/*************************/
/*     VisualVertex      */
/*************************/

//Fully qualified constructor
VisualVertex::VisualVertex(Double_t x, Double_t y, 
  const HepMC::FourVector& position,
  int id, const HepMC::WeightContainer& weights)
: TMarker(x,y,20), GenVertex(position,id,weights){
  
  //Initialize
  Initialize();
}

//Constructor with a GenVertex
VisualVertex::VisualVertex(Double_t x, Double_t y, 
  const HepMC::GenVertex& invertex)
: TMarker(x,y,20), GenVertex(invertex){
  
  //Initialize
  Initialize();

}

////Default initialization for all constructors
void VisualVertex::Initialize(){

  //Proper Marker Size
  SetMarkerSize(2);

  //Default is not protected
  fProtect = kFALSE;

  //Set default label attributes
  fAttLabel = TAttText(13,0,1,0,0.028); //2*fNibbleXRad, but marker size fixed
  fAttLabel.SetTextFont(42);

}

//Default destructor
VisualVertex::~VisualVertex(){

  //Nothing to do so far

}

//
//---- Get and Set methods
//
Bool_t VisualVertex::GetDrawn(){
  
  //Check if I am in the current pad
  if (gPad->FindObject(this)) return kTRUE;
    else return kFALSE;
}

Int_t VisualVertex::GetShownChildren(){

  Int_t NShown=0;

  //Check children for particles that are shown and in the pad
  for ( particle_iterator par = particles_begin(HepMC::children);
    par != particles_end(HepMC::children); ++par ) 
  
    //Check for visual particle
    if (IsVisual(*par))
      //that are actually part of the pad
      if (Visualize(*par)->GetDrawn())
        //Check for shown connections
        if (Visualize(*par)->GetShow())
          //That have a visual vertex
          if (IsVisual((*par)->end_vertex()))
            //that is also sown
            if (Visualize((*par)->end_vertex())->GetDrawn())
              //Count
              NShown++;

  return NShown;

}

Int_t VisualVertex::GetShownParents(){

  Int_t NShown=0;

  //Check parents for particles that are shown and in the pad
  for ( particle_iterator par = particles_begin(HepMC::parents);
    par != particles_end(HepMC::parents); ++par ) 

    //Check for visual particle
    if (IsVisual(*par))
      //that are actually part of the pad
      if (Visualize(*par)->GetDrawn())
        //Check for shown connections
        if (Visualize(*par)->GetShow())
          //That have a visual vertex
          if (IsVisual((*par)->production_vertex()))
            //that is also sown
            if (Visualize((*par)->production_vertex())->GetDrawn())
              //Count
              NShown++;

  return NShown;

}

Double_t VisualVertex::GetNewParentDir(){

  //Allewed range for parents
  Double_t MinAngle=-90;
  Double_t MaxAngle=90;

  //Loop over parents and get their direction
  std::vector<Double_t> dirs;
  dirs.push_back(MinAngle);
  dirs.push_back(MaxAngle);

  //Check parents for particles that are shown and in the pad
  for ( particle_iterator par = particles_begin(HepMC::parents);
    par != particles_end(HepMC::parents); ++par ) {

    //only visual particles
    if (IsVisual(*par)){
      VisualParticle* vpar = Visualize(*par);
      
      //that are already drawn
      if (vpar->GetDrawn() || IsVisual(vpar->production_vertex()))

        //Only take particles that are within the angular range
        if ((vpar->GetDir() > MinAngle) && (vpar->GetDir() < MaxAngle))
          dirs.push_back(vpar->GetDir());
    }
  }
      
  // Sort by direction
  std::sort(dirs.begin(), dirs.end()); 

  Double_t dAngle=0;
  Double_t oAngle=MinAngle;
  
  //Loop to get largest free space
  for (UInt_t iDir = 1; iDir < dirs.size(); iDir++){
    if (dirs.at(iDir)-dirs.at(iDir-1) > dAngle){
      dAngle = dirs.at(iDir)-dirs.at(iDir-1);
      oAngle = dirs.at(iDir-1);
    }
  }

  //Return the middle of the largest bit
  return oAngle+dAngle/2;
  
}

Double_t VisualVertex::GetNewChildDir(){

  //Allewed range for children
  Double_t MinAngle=-90;
  Double_t MaxAngle=90;

  //Loop over children and get their direction
  std::vector<Double_t> dirs;
  dirs.push_back(MinAngle);
  dirs.push_back(MaxAngle);

  //Check children for particles that are shown and in the pad
  for ( particle_iterator par = particles_begin(HepMC::children);
    par != particles_end(HepMC::children); ++par ) {

    //only visual particles
    if (IsVisual(*par)){
      VisualParticle* vpar = Visualize(*par);
      
      //that are already drawn or have a visual end vertex set
      if (vpar->GetDrawn() || IsVisual(vpar->end_vertex()))

        //Only take particles that are within the angular range
        if ((vpar->GetDir() > MinAngle) && (vpar->GetDir() < MaxAngle))
          dirs.push_back(vpar->GetDir());
    }
  }
      
  // Sort by direction
  std::sort(dirs.begin(), dirs.end()); 

  Double_t dAngle=0;
  Double_t oAngle=MinAngle;
  //Loop to get largest free space
  for (UInt_t iDir = 1; iDir < dirs.size(); iDir++){
    if (dirs.at(iDir)-dirs.at(iDir-1) > dAngle){
      dAngle = dirs.at(iDir)-dirs.at(iDir-1);
      oAngle = dirs.at(iDir-1);
    }
  }

  //Return the middle of the largest bit
  return oAngle+dAngle/2;
}

//
//---- Graphics methods
//

Int_t VisualVertex::DistancetoPrimitive(Int_t px, Int_t py){
  
  //Adapted from
  //  Int_t TMarker::DistancetoPrimitive(Int_t px, Int_t py)
  
  // If we are not drawn, the distance is infinite
  if (!GetDrawn()) return 9999;
  
  //  Compute the closest distance of approach from point px,py to this marker.
  //  The distance is computed in pixels units.

  const Int_t kMaxDiff = 8;
  Int_t pxm, pym;
  if (TestBit(kMarkerNDC)) {
     pxm = gPad->UtoPixel(fX);
     pym = gPad->VtoPixel(fY);
  } else {
     pxm  = gPad->XtoAbsPixel(gPad->XtoPad(fX));
     pym  = gPad->YtoAbsPixel(gPad->YtoPad(fY));
  }
  
  Int_t dist = (px-pxm)*(px-pxm) + (py-pym)*(py-pym);
  
  //---changes w/ respect to TMarker:
  //  - user sqrt(dist) instead of dist
  //  - always pick if we are closer than kMaxDiff

  if (TMath::Sqrt(dist) > kMaxDiff) return 9999;
  return 0;
}

void VisualVertex::ExecuteEvent(Int_t event, Int_t px, Int_t py)
{
  // Execute action corresponding to one event.
  // This member function is called when a particle is clicked with the
  // locator

  //Pixel offset for label from cursor location
  const Int_t pXOffset = 10;
  const Int_t pYOffset = 7;

  switch (event) {

    case kButton1Double:
      //Show whole family on double-click
      ToggleAll();
      break ;
    
    case kMouseMotion:
      
      //Redraw pad to remove old labels
      gPad->Draw();
      
      //Draw the vertex label at the mouse pointer
      PaintLabel(px+pXOffset, py+pYOffset);
    
    default :
      
      //Use standard TMarker behaviour
      TMarker::ExecuteEvent(event,px,py);

      //Check that I have not been moved to close to another particle
      if (!gPad) return ;
      //Get all other graphics objects
      TListIter* prims = new TListIter(gPad->GetListOfPrimitives());
      //Loop over primitives and select vertices
      while (TObject* obj = prims->Next()){
        
        //try cast into VisualVertex
        VisualVertex* vvtx = dynamic_cast<VisualVertex*>(obj);

        //If a visual vertex and not ourselves
        if (IsVisual(vvtx)) if (vvtx != this){ 
          Double_t dX = GetX() - vvtx->GetX();
          if (fabs(dX)<0.001) dX=0.001*TMath::Sign(1.,0.5-GetX());
          Double_t dY = GetY() - vvtx->GetY();
          if (fabs(dY)<0.001) dY=0.001*TMath::Sign(1.,0.5-GetY());
          Double_t dist = TMath::Sqrt(dX*dX + dY*dY);
          Double_t mindist = 0.056; //4*fNibbleXRad, but Marker size fixed!
          if (dist < mindist ){ 
            //push me away
            SetX( vvtx->GetX()+ (mindist/dist*dX));
            SetY( vvtx->GetY()+ (mindist/dist*dY));
            gPad->Modified(kTRUE);
          }
        }
      }
    //Make sure we are still on canvas
    if (GetX()<.01) SetX(.01); if (GetY()<.01) SetY(.01);
    if (GetX()>.99) SetX(.99); if (GetY()>.99) SetY(.99);
  }

}

void VisualVertex::Draw(Option_t* option){
  
  //Draw the family:
  // -- vertex itself, all its particles
  // -- children drawn downwards
  // -- parents drawn upwards

  //Draw vertex
  TMarker::Draw(option);

  //Draw children and downwards
  //Particle list may change during loop due to visualization
  // -- use independant vector instead of iterator
  
  std::list<GenParticle*> nonvispar;
  //Get a list of non-visuals
  copy_if( particles_begin(HepMC::family), particles_end(HepMC::family),
           back_inserter(nonvispar), Is_Not_Visual() );
  //Now make all these particles visual
  for ( std::list<GenParticle*>::iterator nvpar = nonvispar.begin();
        nvpar != nonvispar.end(); nvpar++){
     //Make particle visual
     Visualize(*nvpar); 
  }
  
  for ( particle_iterator par = particles_begin(HepMC::children);
    par != particles_end(HepMC::children); ++par ) {
     
    if (!IsVisual(*par)) 
      cerr << "ERROR: Found unexpected non-visual particle!"
           << " -- This indicates a bug!!!" << endl;

    //Cast to visual
    VisualParticle* vpar = Visualize(*par);
    
    //Check if this was marked for reconnect
    if (vpar->GetReconnect()){
      vpar->SetReconnect(kFALSE);
      vpar->SetShow();
      vpar->UpdateChildren();
    }

    //Avoid loops - exclude drawn particles
    if (!vpar->GetDrawn()) {
       
      //Draw particles
      vpar->Draw(option);

      //Auto-unfold if they do not have an end-vertex
  //    if (!((*par)->end_vertex())) 
  //      vpar->SetShow();
      
      //Update family downwards
      if (vpar->GetShow())
        vpar->UpdateChildren();

    }
  }

  //Draw parents and upwards
  for ( particle_iterator par = particles_begin(HepMC::parents);
    par != particles_end(HepMC::parents); ++par ) {

    if (!IsVisual(*par)) 
      cerr << "ERROR: Found unexpected non-visual particle!"
           << " -- This indicates a bug!!!" << endl;
    
    //Cast to visual
    VisualParticle* vpar = Visualize(*par);
    
    //Check if this was marked for reconnect
    if (vpar->GetReconnect()){
      vpar->SetReconnect(kFALSE);
      vpar->SetShow();
      vpar->UpdateParents();
    }

    //Avoid loops - exclude drawn particles
    if (!vpar->GetDrawn()) {

      //Draw particles
      vpar->Draw(option);
      
      //Auto-unfold if they do not have an prod-vertex
  //    if (!((*par)->production_vertex())) 
  //      vpar->SetShow();

      //Update family upwards
      if (vpar->GetShow())
        vpar->UpdateParents();

    }
  }

  //on top of the rest
  TMarker::Pop();
}

void VisualVertex::Paint(Option_t* option){

  //Check that all particles are actually drawn
  //some may have been removed in unfold-action

  //Loop over all shown child particles
  for ( particle_iterator par = particles_begin(HepMC::family);
    par != particles_end(HepMC::family); ++par ){ 
    
    //Only paint visual particles. Newly added GenParticles will be converted 
    //on next VisualVertex::Draw();
    if (IsVisual(*par)) {
    
      VisualParticle* vpar = Visualize(*par);
      
      // if any child or parent is removed accidentaly,
      if (!(vpar->GetDrawn())){
        vpar->SetShow(kFALSE);
        vpar->SetReconnect(kTRUE);
        vpar->Draw();
      }
    }
  }

  //Draw the marker itself
  TMarker::Paint(option);

}

//Paint a small label showing the vertex barcode
void VisualVertex::PaintLabel(Int_t px, Int_t py){

  //Change to current text attributes
  fAttLabel.Modify();
  //Paint the label
  gVirtualX->DrawText(px,py,0,1,Form("VBC: %i",barcode()),TVirtualX::kOpaque);
  //Update the canvas to show it
  gPad->Update();

}

void VisualVertex::UndrawParents(){
  
  //Check for vertex protection
  if (GetProtect()) return ;
  
  //Only proceed if there are no shown children
  if (GetShownChildren() > 0) return;

  //Loop over all shown parent particles
  for ( particle_iterator par = particles_begin(HepMC::parents);
    par != particles_end(HepMC::parents); ++par ) {

    //Only work on visual particles
    if (IsVisual(*par)){

      //Update particles parents if shown
      if (Visualize(*par)->GetShow()){

        //Remove particle before it realizes its vertex is gone
        Visualize(*par)->Undraw();

        //First check for vertex
        if (IsVisual((*par)->production_vertex())){

          //Recursive: if particle has a shown production-vertex,
          VisualVertex* vert = Visualize((*par)->production_vertex());

          //remove those parents as well
          if (vert->GetDrawn())
            vert->UndrawParents();
          //Undraw parent vertex if possible
          vert->Undraw();
        }
      }
    }
  }
}

void VisualVertex::UndrawChildren(){

  //Check for vertex protection
  if (GetProtect()) return ;

  //Only proceed if there are no shown parents
  if (GetShownParents() > 0) return;

  //Loop over all shown child particles
  for ( particle_iterator par = particles_begin(HepMC::children);
    par != particles_end(HepMC::children); ++par ) {

    //Only work on visual particles
    if (IsVisual(*par)){

      //Update particles children if shown
      if (Visualize(*par)->GetShow()){

        //Remove particle before it realizes its vertex is gone
        Visualize(*par)->Undraw();

        //First check for vertex
        if (IsVisual((*par)->end_vertex())){
          
          //Recursive: if particle has a shown end-vertex,
          VisualVertex* vert = Visualize((*par)->end_vertex());

          //remove those children as well
          if (vert->GetDrawn())
            vert->UndrawChildren();
          //Undraw parent vertex if possible
          vert->Undraw();
        }
      }
    }
  }
}

//ToggleAll() and Draw() are the only functions where particles can be
//visualized.
void VisualVertex::ToggleAll(){
//Show/Unshow the whole family

  //Check how many particles are shown
  Int_t NShownParticles=0;
  for ( particle_iterator par = particles_begin(HepMC::family);
    par != particles_end(HepMC::family); ++par ) 
     if (IsVisual(*par))
       if (Visualize(*par)->GetShow())
         NShownParticles++;

  //If all the family is shown, then
  if ( NShownParticles == particles_out_size() + particles_in_size()){
    //Unshow the whole family
    for ( particle_iterator par = particles_begin(HepMC::family);
      par != particles_end(HepMC::family); ++par )
       //Do not visualize in unshow action
       if (IsVisual(*par))
         Visualize(*par)->SetShow(kFALSE);
  
  } else { 
    //Show them and update their relatives
    for ( particle_iterator par = particles_begin(HepMC::family);
      par != particles_end(HepMC::family); ++par ){
       //Make particle visual if it has not been before
       VisualParticle* vpar = Visualize(*par);
       vpar->SetShow(kTRUE);
       vpar->SetReconnect(kFALSE);
       //Force update of relatives/reinitalize of loose relatives by undrawing
       vpar->Undraw();
     }
  }
  
  //Redraw myself for changes to take effect
  Draw();

}

void VisualVertex::Undraw(){

  //As long as there are connected vertices in the familiy
  //the vertex can not be undrawn

  //Only remove if unconnected
  if ((GetShownParents()==0)&&(GetShownChildren()==0)) {
    //and not protected
    if (!GetProtect())
      //Remove myself from the Pad
      gPad->RecursiveRemove(this);
  }
}
  
/*************************/
/*      VisualEvent      */
/*************************/

//Fully qualified constructor
VisualEvent::VisualEvent( int signal_process_id, int event_number,
  HepMC::GenVertex* signal_vertex, const HepMC::WeightContainer& weights,
  const vector<long>& randomstates
//  , HepMC::HeavyIon& ion, HepMC::PdfInfo& pdf
 )
: GenEvent(signal_process_id,event_number,signal_vertex,
            weights,randomstates
//            ,ion,pdf
            )
{

  //nothing else so far
  fpdt=0;

}

//Constructor with a GenEvent
VisualEvent::VisualEvent(const HepMC::GenEvent& inevent)
 : GenEvent (inevent){

 //nothing else so far
 fpdt=0;

}


//
//-- Graphics methods
//

void VisualEvent::Draw(Option_t* option){

  //If there is no signal vertex
  if (!signal_process_vertex()) 
    //take the first one
    set_signal_process_vertex(*vertices_begin());

  //Draw signal vertex
  if (signal_process_vertex()) 
    Visualize(signal_process_vertex())->Draw(option);
}

//
// Particle data access methods
//

//Get Particle Data 
ParticleData* VisualEvent::GetParticleData(Int_t pid){

  //check for table
  if (!fpdt) return 0;

  return fpdt->find(pid);
}

//Get Particle Name
std::string VisualEvent::GetParticleName(Int_t pid){

  //create an empty string
  std::string name;

  //check for table
  if (fpdt)

    //try to find particle 
    if (fpdt->find(pid))

      //Get its name
      name = fpdt->find(pid)->name();

  //return name
  return name;
  
}

/*************************/
/*     Global methods    */
/*************************/

Bool_t Is_Visual::operator()( HepMC::GenVertex* invertex ){
  
  //Sanitiy check
  if (!invertex) return kFALSE;
  
//  using namespace ROOT::Reflex;

  //Check if vertex is visual
  if ( typeid(*invertex) == typeid(VisVertexType) )
    return kTRUE;
  else return kFALSE;
}

Bool_t Is_Visual::operator()( HepMC::GenParticle* inparticle ){

  //Sanitiy check
  if (!inparticle) return kFALSE;
  
//  using namespace ROOT::Reflex;
  //Check if vertex is visual
  if ( typeid(*inparticle) == typeid(VisParticleType) )
    return kTRUE;
  else return kFALSE;
}

Bool_t Is_Visual::operator()( HepMC::GenEvent* inevent ){
  
  //Sanitiy check
  if (!inevent) return kFALSE;
  
//  using namespace ROOT::Reflex;
  //Check if vertex is visual
  if ( typeid(*inevent) == typeid(VisEventType))
    return kTRUE;
  else return kFALSE;
}

///@cond NEVER
//Make particle iterator global, but do not export in reflex dictionary
//and do not show in doxygen
typedef GenVertex::particle_iterator particle_iterator ;
typedef GenVertex::particles_in_const_iterator particles_in_iterator ;
///@endcond

///Convert GenVertex to VisualVertex. Some effort is put into finding a good
///default position of the vertex on the canvas. All particles are stripped form
///the GenVertex and added to the new VisualVertex. Any GenParticles contained
///by the GenVertex will not be converted but added as GenParticles to the new
///vertex.
VisualVertex* HepMC::Visualize( HepMC::GenVertex* invertex ){
  
  //sanity check
  if (!invertex) return 0;
  
  //Check if vertex is visual
  if (IsVisual(invertex)) return dynamic_cast<VisualVertex*>(invertex) ;

  //Check if we have any visual vertices around us that may provide a 
  //canvas position
  Double_t DefaultLength 
    = VisParticleType.GetDefaultLength()*1.5;
  Double_t x=0; Double_t y=0;
  Double_t theta=0;
  Int_t NVisVertex=0;
  //Loop over parents
  for ( particle_iterator par =invertex->particles_begin(HepMC::parents);
        par != invertex->particles_end(HepMC::parents); par++ ){
    GenVertex* parv = (*par)->production_vertex();
    //that may provide a production vertex
    if ( parv ){
      //If this one is visible
      if ( IsVisual(parv) ) {
        VisualVertex* visparv = dynamic_cast<VisualVertex*>(parv);
        //get average y
        y += visparv->GetY();
        //get rightmost x
        x = (visparv->GetX() > x) ? visparv->GetX() : x ;
        //If this particle is already shown
        if (IsVisual(*par)){
          VisualParticle* vpar = Visualize(*par);
          if (vpar->GetDrawn())
            //get the particle direction
            theta = vpar->GetDir();
        } else 
          //get new free direction
          theta=visparv->GetNewChildDir();
        //count
        NVisVertex++;
      }
    }
  }
  //If we can get a position from some parent
  if (NVisVertex > 0 ){
    //Get position from parent(s?)
    if (NVisVertex > 1 ){
      //We have several parents - take average y and rightmost x + offset
      y = y/NVisVertex;
      x = x + DefaultLength;
    } else {
      //We only have a single vertex -> use the NewChildDir();
      x = x + DefaultLength*TMath::Cos(theta/180.*TMath::Pi());
      y = y + DefaultLength*TMath::Sin(theta/180.*TMath::Pi());
    }
  } else {
    //Try the children
    x = 1;
    //Loop over children
    for ( particle_iterator par = invertex->particles_begin(HepMC::children);
          par != invertex->particles_end(HepMC::children); par++ ){
        GenVertex* childv = (*par)->end_vertex();
        //that may provide a production vertex
        if ( childv )
          //If this one is visible
          if ( IsVisual(childv) ) {
            VisualVertex* vischildv = dynamic_cast<VisualVertex*>(childv);
            //get average y
            y += vischildv->GetY();
            //get leftmost x
            x = (vischildv->GetX() < x) ? vischildv->GetX() : x ;
            //If particle is already shown
            if (IsVisual(*par)){
              VisualParticle* vpar = Visualize(*par);
              if (vpar->GetDrawn())
                //get the particle direction
                theta = vpar->GetDir();
            } else 
              //get free directionn
              theta=vischildv->GetNewParentDir();
            //count
            NVisVertex++;
            }
    }
    //If we can get a vertex position from some child
    if (NVisVertex > 0){
      //Get vertex position from parent(s?)
      if (NVisVertex > 1 ){
        //We have several children - take average y and leftmost x - offset
        y = y/NVisVertex;
        x = x - DefaultLength;
      } else {
        //We only have a single vertex -> use the NewParentDir();
        x = x - DefaultLength*TMath::Cos(theta/180.*TMath::Pi());
        y = y - DefaultLength*TMath::Sin(theta/180.*TMath::Pi());
      }
    } else {
      //Last resort - set default vertex location
      x = .5 ; y= .5 ;
    }
  }

  //Try to avoid hitting other vertices

  //Get all other graphics objects
  if (gPad){
    TListIter* prims = new TListIter(gPad->GetListOfPrimitives());
    //Loop over primitives and select vertices
    while (TObject* obj = prims->Next()){
      
      //try cast into VisualVertex
      VisualVertex* vvtx = dynamic_cast<VisualVertex*>(obj);

      //If a visual vertex
      if (IsVisual(vvtx)){ 
        Double_t dX = x - vvtx->GetX();
        if (fabs(dX)<0.001) dX=0.001*TMath::Sign(1.,0.5-x);
        Double_t dY = y - vvtx->GetY();
        if (fabs(dY)<0.001) dY=0.001*TMath::Sign(1.,0.5-y);
        Double_t dist = TMath::Sqrt(dX*dX + dY*dY);
        Double_t mindist = 0.056; //4*fNibbleXRad, but Marker size fixed!
        if (dist < mindist ){ 
          //push me away
          x = vvtx->GetX()+ (mindist/dist*dX);
          y = vvtx->GetY()+ (mindist/dist*dY);
        }
      }
    }
  }

  //Make sure we are still on canvas
  if (x<.01) x=.01; if (y<.01) y=.01; if (x>.99) x=.99; if (y>.99) y=.99;

  //Create a new visual vertex at the new position
  // - we can not use VisualVertex(GenVertex&),
  //   as this will copy particles as well
  VisualVertex* newvertex = new VisualVertex(x,y,FourVector(0,0,0,0),
  invertex->id(), invertex->weights());
  newvertex->set_position(invertex->position());

  // Add all references to in- and out- particles
  // - this will this detach the particle from the invertex
  
  //Can not use normal particle iterator, as particle list
  //will change during the loop.
  // - Loop over first parents until none is left
  for ( particle_iterator par=invertex->particles_begin(HepMC::parents);
        invertex->particles_in_size() > 0; 
        par= invertex->particles_begin(HepMC::parents) ) {
    //Add newvertex as end_vertex
    newvertex->add_particle_in(*par);
  }
  //Loop over children
  for ( particle_iterator par=invertex->particles_begin(HepMC::children);
        invertex->particles_out_size() > 0; 
        par = invertex->particles_begin(HepMC::children)){
    //Add newvertex as production_vertex
    newvertex->add_particle_out(*par);
  }

  //Check if this was part of an event
  GenEvent* evt = invertex->parent_event();
  if (evt){
    //replace if this was the signal vertex of the event
    if (evt->signal_process_vertex() == invertex )
      evt->set_signal_process_vertex(newvertex);

    //First detach invertex from its event
    // - otherwise, the new vertex gets a different barcode
    evt->remove_vertex(invertex);
    
    //Now assign old barcode
    newvertex->suggest_barcode( invertex->barcode() );

    //plugin new vertex to event
    evt->add_vertex(newvertex);
  }

  return newvertex;
}

///Convert a GenParticle into a VisualParticle. A new VisualParticle is created
///and put in place of the old GenParticle, i.e. replaced at the production
///vertex and end vertex.
VisualParticle* HepMC::Visualize( HepMC::GenParticle* inparticle ){
  
  //sanity check
  if (!inparticle) return 0;
  
//  using namespace ROOT::Reflex;
  
  //Check if particle is visual
  if (IsVisual(inparticle))
    return dynamic_cast<VisualParticle*>(inparticle) ;
 
  //Create a new visual particle 
  //  - using VisualParticle(GenParticle&) as this
  //  - will not copy vertices (compare Visualize(GenVertex))
  VisualParticle* newparticle = new VisualParticle(*inparticle);

  //Get the vertices
  HepMC::GenVertex* prodv = inparticle->production_vertex();
  HepMC::GenVertex* endv = inparticle->end_vertex();

  //First detach particle at both vertices, other barcode will change
  if (prodv)prodv->remove_particle(inparticle);
  if (endv) endv->remove_particle(inparticle);
  
  //Then add new particle
  if (prodv) prodv->add_particle_out(newparticle);
  if (endv) endv->add_particle_in(newparticle);
 
  //Show a warning message if barcode has changed
  if (newparticle->barcode() != inparticle->barcode())
    cout << "HepMCVisual:\tWARNING\t"
         << "Barcode of particle "
         << inparticle->barcode() 
         << " was changed to "
         << newparticle->barcode()
         << endl;

  //event is handled through vertices
  // - no need to do anything here
  return newparticle;

}

///Convert a GenVertex into a VisualVertex. Only a deep copy of the event is
///made, still using GenVertex members. The signal vertex will be automatically
///transformed in calling VisualEvent::Draw().
VisualEvent* HepMC::Visualize( HepMC::GenEvent* inevent){

  //sanity check
  if (!inevent) return 0;
  
//  using namespace ROOT::Reflex;
  
  //Check if vertex is visual
  if (IsVisual(inevent))
    return dynamic_cast<VisualEvent*>(inevent) ;
  
  //We can not properly position all vertices
  //Only make a deep copy of full event using GenEvent(&GenEvent)
  VisualEvent * newevent = new VisualEvent(*inevent);

  //Production vertex will automatically be visualized by GenEvent::Draw()

  return newevent;

}
